BastasTools = LibStub("AceAddon-3.0"):NewAddon("BastasTools", "AceConsole-3.0", "AceEvent-3.0")



--[[----------------------------------------------------------------------------
Defaults
------------------------------------------------------------------------------]]
local defaults = {
	global = {
		nameFilterStr="",
		tradeFilterStr="",
		enableFilters=true,
		debugFilters=false,
		filteredLog = {},
		
		enableAutoMathWhispers=true,
		autoMathGuildAndOfficer=true,
		autoMathSay=true,
		autoMathTradeAndGeneral=false,
		autoMathRaidAndInstance=false,
		autoMathSilent=true,
		
		enableChatForwarder=false,
		forwardTarget="officer",
		
		enableShadowPriestTranslator=true,
		
		enableElvGlow=false,
		showGlowOptionsPanel=false
	}
}



--[[----------------------------------------------------------------------------
Options
------------------------------------------------------------------------------]]
local options = {
	name = "Bastas Tools",
	handler = BastasTools,
	childGroups = "tab",
	type = "group",
	args = {
		SilentMute = {
			name = "Silent Mute",
			desc = "Silently block trade/general/whisper messages by keyword or player name.",
			type = "group",
			childGroups  = "tab",
			args = {
				openFilterOptions = {
					name = "Options",
					desc = "Setup filtering options.",
					type = "group",
					args = {
						nameFilters = {
							name = "Name Filters",
							desc = "Block messages from players on this list. Put one name per line. Case insensitive. Including the realm is optional. Example: Player-Realm or just Player",
							type = "input",
							order = 3,
							multiline = 20,
							get = function(info) return BastasTools.db.global.nameFilterStr end,
							set = function(info,val) BastasTools.db.global.nameFilterStr = val end
						},
						tradeFilters = {
							name = "Trade/General Chat Filters",
							desc = "Block text containing keywords or phrases. Case insensitive. Put one keyword/phrase per line.",
							type = "input",
							order = 4,
							multiline = 20,
							get = function(info) return BastasTools.db.global.tradeFiltersStr end,
							set = function(info,val) BastasTools.db.global.tradeFiltersStr = val end
						},
						enableFilters = {
							name = "Enabled",
							desc = "Allow the silent mute module to filter out annoying messages.",
							type = "toggle",
							order = 1,
							width = "full",
							get = function(info) return BastasTools.db.global.enableFilters end,
							set = function(info,val) BastasTools.db.global.enableFilters = val end
						},
						debugFilters = {
							name = "Debug Mode",
							desc = "When a message is filtered, display debug information.",
							type = "toggle",
							order = 2,
							width = "full",
							get = function(info) return BastasTools.db.global.debugFilters end,
							set = function(info,val) BastasTools.db.global.debugFilters = val end
						}
					}
				},				
				openFilterLog = {
					name = "Log",
					desc = "View the 100 most-recently blocked messages",
					type = "group",
					args = {
						filterLog = {
							name = "Blocked Messages",
							desc = "The 100 most-recently blocked messages.",
							type = "input",
							width = "full",
							multiline = 25,
							disabled = true,
							descStyle = "inline",
							get = function(info) 
								local txt = table.concat(BastasTools.db.global.filteredLog,"\n");
								if ( txt and txt ~= "" ) then return txt end;
								return "no messages in log.";
							end
						},
						clearLog = {
							name = "Clear",
							desc = "Clear log of blocked messages.",
							type = "execute",
							func = function(info) 
								BastasTools:ClearFilterLog() 
							end
						}
					}
				}
			}
		},
		AutoMath = {
			name = "Auto Math",
			desc = "Automatically compute solutions to math expression in chat messages.",
			type = "group",
			args = {
				enableAutoMathWhispers = {
					name = "Whispers",
					desc = "Enable the auto computing of math expressions found in private chat messages.",
					type = "toggle",
					order = 1,
					width = "full",
					get = function(info) return BastasTools.db.global.enableAutoMathWhispers end,
					set = function(info,val) BastasTools.db.global.enableAutoMathWhispers = val end
				},
				autoMathTradeAndGeneral = {
					name = "Trade / General",
					desc = "Enable the auto computing of math expressions found in trade / general chat messages.",
					type = "toggle",
					order = 2,
					width = "full",
					get = function(info) return BastasTools.db.global.autoMathTradeAndGeneral end,
					set = function(info,val) BastasTools.db.global.autoMathTradeAndGeneral = val end
				},
				autoMathRaidAndInstance = {
					name = "Raid / Instance",
					desc = "Enable the auto computing of math expressions found in raid / instance chat messages.",
					type = "toggle",
					order = 3,
					width = "full",
					get = function(info) return BastasTools.db.global.autoMathRaidAndInstance end,
					set = function(info,val) BastasTools.db.global.autoMathRaidAndInstance = val end
				},
				autoMathGuildAndOfficer = {
					name = "Guild / Officer",
					desc = "Enable the auto computing of math expressions found in guild / officer chat messages.",
					type = "toggle",
					order = 4,
					width = "full",
					get = function(info) return BastasTools.db.global.autoMathGuildAndOfficer end,
					set = function(info,val) BastasTools.db.global.autoMathGuildAndOfficer = val end
				},
				autoMathSay = {
					name = "Say / Yell",
					desc = "Enable the auto computing of math expressions found local say / yell chat messages.",
					type = "toggle",
					order = 5,
					width = "full",
					get = function(info) return BastasTools.db.global.autoMathSay end,
					set = function(info,val) BastasTools.db.global.autoMathSay = val end
				},
				autoMathSilent = {
					name = "Quiet Mode",
					desc = "If checked, answers will be printed locally instead of being automatically sent in a chat message.",
					type = "toggle",
					order = 6,
					width = "full",
					get = function(info) return BastasTools.db.global.autoMathSilent end,
					set = function(info,val) BastasTools.db.global.autoMathSilent = val end
				}

			}
		},
		ChatForwarder = {
			name = "Chat Forwarder",
			desc = "Forward chat message to another player or chat channel.",
			type = "group",
			args = {
				enableChatForwarder = {
					name = "Enabled",
					desc = "Enable the forwarding of chat messages.",
					type = "toggle",
					order = 1,
					get = function(info) return BastasTools.db.global.enableChatForwarder end,
					set = function(info,val) BastasTools.db.global.enableChatForwarder = val end
				},
				forwardTarget = {
					name = "Forward Target",
					desc = "Player-realm to forward to a player. Guild to forward to guild chat. Officer to forward to guild officer chat.",
					type = "input",
					order = 2,
					get = function(info) return BastasTools.db.global.forwardTarget end,
					set = function(info,val) BastasTools.db.global.forwardTarget = val end
				}
			},
		},
		ShadowPriestTranslator = {
			name = "Voidform Translator",
			desc = "While in a raid group will translate voidform messages from shadow priests. Requires both sender and receiver of messages to have this addon. NOTE: Due to current API limitations this does not work for chat bubbles.",
			type = "group",
			args = {
				enableShadowPriestTranslator = {
					name = "Enabled",
					desc = "Enable the translation of voidform messages among users of this addon.",
					type = "toggle",
					get = function(info) return BastasTools.db.global.enableShadowPriestTranslator end,
					set = function(info,val) BastasTools.db.global.enableShadowPriestTranslator = val end
				},
			},		
		},
		ElvGlow = {
			name = "ElvUI Frame Highlights",
			desc = "Additional highlighting options for ElvUI Raid frames.",
			type = "group",
			args = {
				enableElvGlow = {
					name = "Enabled",
					desc = "Enable additional highlighting options for ElvUI Raid Frames. Requires ElvUI frames.",
					type = "toggle",
					get = function(info) return BastasTools.db.global.enableElvGlow end,
					set = function(info,val) BastasTools.db.global.enableElvGlow = val end
				},
				showElvGlowOptionsPanel = {
					name = "Options Panel",
					desc = "Show options panel for this feature.",
					type = "toggle",
					get = function(info) return BastasTools.db.global.showGlowOptionsPanel end,
					set = function(info,val) BastasTools.db.global.showGlowOptionsPanel = val; BastasTools:UpdateElvGlowOptionsVisibility(); end
				}
			},
		}
		
	}
}



--[[----------------------------------------------------------------------------
Handle Chat Commands
------------------------------------------------------------------------------]]
function BastasTools:ChatCommand(input)
    if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	elseif ( string.lower(input) == "glow" ) then
		BastasTools:ToggleElvGlowMenu()
	end
end



--[[----------------------------------------------------------------------------
Addon Initialized
------------------------------------------------------------------------------]]
function BastasTools:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("BastasToolsDB", defaults)

	LibStub("AceConfig-3.0"):RegisterOptionsTable("BastasTools",options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("BastasTools", "BastasTools")
	self:RegisterChatCommand("bt","ChatCommand")
	
	if ( self.InitAutoMath ) then
		self:InitAutoMath()
	end
	
	if ( self.InitElvGlow ) then
		self:InitElvGlow()
	end
end

